@@ -2,6 +2,11 @@
 #'
 #' Approximate string matching equivalents of \code{R}'s native \code{\link[base]{match}} and \code{\%in\%}.
 #'
+#' \code{ain} is currently defined as 
+#' 
+#' \code{ain(x,table,...) <- function(x,table,...) amatch(x, table, nomatch=0,...) > 0}
+#'
+#'
 #' @section Note on \code{NA} handling:
 #' \code{R}'s native \code{\link[base]{match}} function matches \code{NA} with \code{NA}. This
 #' may feel inconsistent with \code{R}'s usual \code{NA} handling, since for example \code{NA==NA} yields
@@ -18,15 +23,17 @@
 #' is \code{NA}, the \code{nomatch} value is returned, regardless of whether \code{NA} is present in \code{table}.
 #'
 #'
-#' @param x vector: elements to be approximately matched
-#' @param table vector: lookup table for matching
-#' @param nomatch the value to be returned when no match is found. This is coerced to integer. \code{nomatch=0} 
+#' @param x vector: elements to be approximately matched: will be coerced to \code{character}.
+#' @param table vector: lookup table for matching. Will be coerced to \code{character}.
+#' @param nomatch The value to be returned when no match is found. This is coerced to integer. \code{nomatch=0} 
 #'  can be a useful option.
 #' @param matchNA Should \code{NA}'s be matched? Default behaviour mimics the
 #'   behaviour of base \code{\link[base]{match}}, meaning that \code{NA} matches
-#'   \code{NA} (which is inconsistent with \code{dist} or \code{stringdist}).
+#'   \code{NA} (see also the note on \code{NA} handling below).
 #' @param method Matching algorithm to use. See \code{\link{stringdist}}.
-#' @param weight parameters for matching algorithm See \code{\link{stringdist}}.
+#' @param useBytes Perform byte-wise comparison. \code{useBytes=TRUE} is faster but may yield different
+#' 	results depending on character encoding. See also \code{\link{stringdist}}, under encoding issues.
+#' @param weight Weight parameters for matching algorithm See \code{\link{stringdist}}.
 #' @param maxDist Elements in \code{x} will not be matched with elements of
 #'  \code{table} if their distance is larger than \code{maxDist}. 
 #'   
@@ -41,6 +48,7 @@
 #' @export
 amatch <- function(x, table, nomatch=NA_integer_, matchNA=TRUE, 
   method=c("osa","lv","dl","hamming","lcs","qgram","cosine","jaccard", "jw"), 
+  useBytes = FALSE,
   weight=c(d=1,i=1,s=1,t=1), 
   maxDist=0.1, q=1, p=0){
 
@@ -49,17 +57,20 @@ amatch <- function(x, table, nomatch=NA_integer_, matchNA=TRUE,
   table <- as.character(table)
 
   method <- match.arg(method)
-  x <- char2int(x)
-  table <- char2int(table)
+  if (!useBytes){
+    x <- char2int(x)
+    table <- char2int(table)
+  }
   stopifnot(
-      all(is.finite(weight)),
-      all(weight > 0),
-      all(weight <=1),
-      q >= 0,
-      p <= 0.25,
-      p >= 0,
-      matchNA %in% c(TRUE,FALSE),
-      maxDist > 0
+      all(is.finite(weight))
+      , all(weight > 0)
+      , all(weight <=1)
+      , q >= 0
+      , p <= 0.25
+      , p >= 0
+      , matchNA %in% c(TRUE,FALSE)
+      , maxDist > 0
+      , is.logical(useBytes)
   )
   if (maxDist==Inf) maxDist <- 0L;
   switch(method,
@@ -67,15 +78,17 @@ amatch <- function(x, table, nomatch=NA_integer_, matchNA=TRUE,
     lv      = .Call('R_match_lv'        , x, table, as.integer(nomatch), as.integer(matchNA), as.double(weight), as.double(maxDist)),
     dl      = .Call('R_match_dl'        , x, table, as.integer(nomatch), as.integer(matchNA), as.double(weight), as.double(maxDist)),
     hamming = .Call('R_match_hm'        , x, table, as.integer(nomatch), as.integer(matchNA), as.integer(maxDist)),
-    lcs     = .Call('R_match_hm'        , x, table, as.integer(nomatch), as.integer(matchNA), as.integer(maxDist)),
+    lcs     = .Call('R_match_lcs'        , x, table, as.integer(nomatch), as.integer(matchNA), as.integer(maxDist)),
     qgram   = .Call('R_match_qgram_tree', x, table, as.integer(nomatch), as.integer(matchNA), as.integer(q), as.double(maxDist), 0L),
     cosine  = .Call('R_match_qgram_tree', x, table, as.integer(nomatch), as.integer(matchNA), as.integer(q), as.double(maxDist), 1L),
     jaccard = .Call('R_match_qgram_tree', x, table, as.integer(nomatch), as.integer(matchNA), as.integer(q), as.double(maxDist), 2L),
-    jw      = .Call('R_match_jw'        , x, table, as.integer(nomatch), as.integer(matchNA), as.double(p), as.double(maxDist))
+    jw      = .Call('R_match_jw'        , x, table, as.integer(nomatch), as.integer(matchNA), as.double(p), as.double(weight), as.double(maxDist))
   )
 }
 
 #' @param ... parameters to pass to \code{amatch} (except \code{nomatch})
+#'
+#'
 #' @rdname amatch
 #' @export 
 ain <- function(x,table,...){
