@@ -1,4 +1,23 @@
-/* This code is gratefully based on Nick Logan's github repository
+/*  stringdist - a C library of string distance algorithms with an interface to R.
+ *  Copyright (C) 2013  Mark van der Loo
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
+ *
+ *  You can contact the author at: mark _dot_ vanderloo _at_ gmail _dot_ com
+ *
+ *
+ * This code is gratefully based on Nick Logan's github repository
  * https://github.com/ugexe/Text--Levenshtein--Damerau--XS/blob/master/damerau-int.c
  * 
  *
@@ -7,13 +26,13 @@
  * - Added R interface function
  * - Added edit weights (function is now of type double)
  * - Added corner cases for length-zero strings.
- * - Replaced linked list dictionary with fixed-size struct for loop externalization.
+ * - Replaced linked list dictionary with fixed-size struct for loop 
+ *    externalization of memory allocation.
  * - Externalized allocation of dynamic programming matrix.
  * 
- * mark.vanderloo@gmail.com
+ * 
  */
 
-/* ugexe@cpan.org (Nick Logan)    */
 
 #define USE_RINTERNALS
 #include <R.h>
@@ -93,14 +112,14 @@ static double distance(
       double *scores
     ){
 
-  if (x == 0){
+  if (!x){
     if ( maxDistance > 0 && maxDistance < y ){
       return -1;
     } else {
       return (double) y;
     }
   }
-  if (y == 0){
+  if (!y){
     if (maxDistance > 0 && maxDistance < x){
       return -1;
     } else {
@@ -142,10 +161,10 @@ static double distance(
         subScore = scores[i * (y + 2) + j] + weight[2];
         insScore = scores[(i+1) * (y + 2) + j] + weight[1];
         delScore = scores[i * (y + 2) + (j + 1)] + weight[0];
-        scores[(i+1) * (y + 2) + (j + 1)] = min2(swapScore, min3(delScore, insScore, subScore));
+        scores[(i+1) * (y + 2) + (j + 1)] = MIN(swapScore, MIN(delScore, MIN(insScore, subScore) ));
       } else {
         swapCount = j;
-        scores[(i+1) * (y + 2) + (j + 1)] = min2(scores[i * (y + 2) + j], swapScore);
+        scores[(i+1) * (y + 2) + (j + 1)] = MIN(scores[i * (y + 2) + j], swapScore);
       }
     }
     /* We will return -1 here if the */
@@ -170,41 +189,52 @@ SEXP R_dl(SEXP a, SEXP b, SEXP weight, SEXP maxDistance){
   PROTECT(maxDistance);
   PROTECT(weight);
    
-  int i=0, j=0, k=0;
-  int na = length(a);
-  int nb = length(b);
-  int nt = (na > nb) ? na : nb;
+  int na = length(a)
+    , nb = length(b)
+    , nt = (na > nb) ? na : nb
+    , bytes = IS_CHARACTER(a)
+    , ml_a = max_length(a)
+    , ml_b = max_length(b);
+  
+
   double maxDist = REAL(maxDistance)[0];
   double *w = REAL(weight);
 
+
+  /* claim space for workhorse */
+
+  dictionary *dict = new_dictionary( ml_a + ml_b + 1 );
+  double *scores = (double *) malloc( (ml_a + 3) * (ml_b + 2) * sizeof(double) );
+
+  unsigned int *s=NULL, *t=NULL;
+  if ( bytes ){
+    s = (unsigned int *) malloc((ml_a + ml_b) * sizeof(int));
+    t = s + ml_a;
+  }
+  if ( (scores == NULL) | (bytes && s == NULL) ){
+    UNPROTECT(4);
+    free(scores);
+    free(s);
+    error("Unable to allocate enough memory");
+  }
+
+
+  // output
   SEXP yy; 
   PROTECT(yy = allocVector(REALSXP, nt));
   double *y = REAL(yy);
 
-  /* claim space for workhorse */
-  int max_a = max_length(a);
-  int max_b = max_length(b);
-  dictionary *dict = new_dictionary( max_length(a) + max_length(b) + 1 );
-  double *scores = (double *) malloc( (max_a + 3) * (max_b + 2) * sizeof(double) );
-  if ( scores == NULL ){
-    UNPROTECT(4);
-    error("%s\n","unable to allocate enough memory");
-  }
-
+  int i=0, j=0, k=0, len_s, len_t, isna_s, isna_t;
   for ( k=0; k < nt; ++k ){
-    if ( INTEGER(VECTOR_ELT(a,i))[0] == NA_INTEGER || INTEGER(VECTOR_ELT(b,j))[0] == NA_INTEGER){
+    s = get_elem(a, i, bytes, &len_s, &isna_s, s);
+    t = get_elem(b, j, bytes, &len_t, &isna_t, t);
+    if ( isna_s || isna_t ){
       y[k] = NA_REAL;
       continue;
     }
     y[k] = distance(
-     (unsigned int *) INTEGER(VECTOR_ELT(a,i)),
-     (unsigned int *) INTEGER(VECTOR_ELT(b,j)),
-      length(VECTOR_ELT(a,i)),
-      length(VECTOR_ELT(b,j)),
-      w,
-      maxDist,
-      dict,
-      scores
+     s, t, len_s, len_t,
+     w, maxDist, dict, scores
     );
     if (y[k] < 0 ) y[k] = R_PosInf;
     i = RECYCLE(i+1,na);
@@ -213,6 +243,9 @@ SEXP R_dl(SEXP a, SEXP b, SEXP weight, SEXP maxDistance){
   
   free_dictionary(dict);
   free(scores);
+  if ( bytes ){
+    free(s);
+  } 
   UNPROTECT(5);
   return yy;
 } 
@@ -227,60 +260,58 @@ SEXP R_match_dl(SEXP x, SEXP table, SEXP nomatch, SEXP matchNA, SEXP weight, SEX
   PROTECT(weight);
   PROTECT(maxDistance);
 
-  int nx = length(x), ntable = length(table);
-  int no_match = INTEGER(nomatch)[0];
-  int match_na = INTEGER(matchNA)[0];
+  int nx = length(x)
+    , ntable = length(table)
+    , no_match = INTEGER(nomatch)[0]
+    , match_na = INTEGER(matchNA)[0]
+    , bytes = IS_CHARACTER(x)
+    , ml_x = max_length(x)
+    , ml_t = max_length(table);
+
   double *w = REAL(weight);
   double maxDist = REAL(maxDistance)[0];
   
   /* claim space for workhorse */
-  int max_x = max_length(x);
-  int max_table = max_length(table);
-  dictionary *dict = new_dictionary( max_x + max_table + 1 );
-  double *scores = (double *) malloc( (max_x + 3) * (max_table + 2) * sizeof(double) );
-  if ( scores == NULL ){
-    UNPROTECT(6);
-    error("%s\n","unable to allocate enough memory");
+  dictionary *dict = new_dictionary( ml_x + ml_t + 1 );
+  double *scores = (double *) malloc( (ml_x + 3) * (ml_t + 2) * sizeof(double) );
+
+  unsigned int *X = NULL, *T = NULL;
+  if ( bytes ){
+    X = (unsigned int *) malloc( (ml_x + ml_t) * sizeof(int) );
+    T = X + ml_x;
+  }
+
+  if ( (scores == NULL) | (bytes && X == NULL) ){
+    UNPROTECT(6); free(X); 
+    error("Unable to allocate enough memory");
   }
 
   // output vector
   SEXP yy;
   PROTECT(yy = allocVector(INTSXP, nx));
   int *y = INTEGER(yy);
-  int *X, *T;
-
 
   double d = R_PosInf, d1 = R_PosInf;
-  int index, xNA, tNA;
+  int index, len_X, len_T, isna_X, isna_T;
 
   for ( int i=0; i<nx; i++){
     index = no_match;
-
-    X = INTEGER(VECTOR_ELT(x,i));
-    xNA = (X[0] == NA_INTEGER);
+    X = get_elem(x, i , bytes, &len_X, &isna_X, X);
+    d1 = R_PosInf;
 
     for ( int j=0; j<ntable; j++){
+      T = get_elem(table, j, bytes, &len_T, &isna_T, T);
 
-      T = INTEGER(VECTOR_ELT(table,j));
-      tNA = (T[0] == NA_INTEGER);
-
-      if ( !xNA && !tNA ){        // both are char (usual case)
+      if ( !isna_X && !isna_T ){        // both are char (usual case)
         d = distance(
-          (unsigned int *) X,
-          (unsigned int *) T,
-          length(VECTOR_ELT(x,i)),
-          length(VECTOR_ELT(table,j)),
-          w,
-          maxDist,
-          dict,
-          scores
+          X, T, len_X, len_T, w, maxDist, dict, scores
         );
         if ( d > -1 && d < d1){ 
           index = j + 1;
           if ( abs(d) < 1e-14 ) break;
           d1 = d;
         }
-      } else if ( xNA && tNA ) {  // both are NA
+      } else if ( isna_X && isna_T ) {  // both are NA
         index = match_na ? j + 1 : no_match;
         break;
       }
@@ -289,6 +320,7 @@ SEXP R_match_dl(SEXP x, SEXP table, SEXP nomatch, SEXP matchNA, SEXP weight, SEX
     y[i] = index;
   }  
   UNPROTECT(7);
+  if (bytes) free(X);
   free_dictionary(dict);
   free(scores);
   return(yy);
